searchData={"content_type":"text/markdown","items":[{"doc":"","ref":"Mix.Tasks.Compile.Temple.html","title":"mix compile.temple","type":"task"},{"doc":"This task is useful for converting a ton of HTML into Temple syntax.\n\n> #### Note about EEx and HEEx {: .tip}\n>\n> In the future, this should be able to convert EEx and HEEx as well, but that would involve invoking or forking their parsers. That is certainly doable, but is out of scope for what I needed right now. Contributions are welcome!","ref":"Mix.Tasks.Temple.Convert.html","title":"mix temple.convert","type":"task"},{"doc":"```shell\n$ mix temple.convert some_file.html\n```","ref":"Mix.Tasks.Temple.Convert.html#module-usage","title":"Usage - mix temple.convert","type":"task"},{"doc":"Temple syntax is available inside the `temple`, and is compiled into efficient Elixir code at compile time using the configured `EEx.Engine`.\n\nYou should checkout the [guides](https://hexdocs.pm/temple/your-first-template.html) for a more in depth explanation.","ref":"Temple.html","title":"Temple","type":"module"},{"doc":"```elixir\ndefmodule MyApp.HomePage do\n  import Temple\n\n  def render() do\n    assigns = %{title: \"My Site | Sign Up\", logged_in: false}\n\n    temple do\n      \"<!DOCTYPE html>\"\n\n      html do\n        head do\n          meta charset: \"utf-8\"\n          meta http_equiv: \"X-UA-Compatible\", content: \"IE=edge\"\n          meta name: \"viewport\", content: \"width=device-width, initial-scale=1.0\"\n          link rel: \"stylesheet\", href: \"/css/app.css\"\n\n          title do: @title\n        end\n\n        body do\n          header class: \"header\" do\n            ul do\n              li do\n                a href: \"/\", do: \"Home\"\n              end\n              li do\n                if @logged_in do\n                  a href: \"/logout\", do: \"Logout\"\n                else\n                  a href: \"/login\", do: \"Login\"\n                end\n              end\n            end\n          end\n\n          main do\n            \"Hi! Welcome to my website.\"\n          end\n        end\n      end\n    end\n  end\nend\n```","ref":"Temple.html#module-usage","title":"Usage - Temple","type":"module"},{"doc":"","ref":"Temple.html#module-configuration","title":"Configuration - Temple","type":"module"},{"doc":"By default Temple wil use the `EEx.SmartEngine`, but you can configure it to use any other engine. Examples could be `Phoenix.HTML.Engine` or `Phoenix.LiveView.Engine`.\n\n```elixir\nconfig :temple, engine: Phoenix.HTML.Engine\n```","ref":"Temple.html#module-engine","title":"Engine - Temple","type":"module"},{"doc":"You can add an alias for an element if there is a namespace collision with a function. If you are using `Phoenix.HTML`, there will be namespace collisions with the ` ` and ` ` elements.\n\n```elixir\nconfig :temple, :aliases,\n  label: :label_tag,\n  link: :link_tag,\n  select: :select_tag\n\ntemple do\n  label_tag do\n    \"Email\"\n  end\n\n  link_tag href: \"/css/site.css\"\nend\n```\n\nThis will result in:\n\n```html\n \n  Email\n \n\n \n```","ref":"Temple.html#module-aliases","title":"Aliases - Temple","type":"module"},{"doc":"Compiles runtime attributes.\n\nTo use this function, you set it in application config.\n\nBy default, Temple uses `{Phoenix.HTML, :attributes_escape}`. This is useful if you want to use `EEx.SmartEngine`.\n\n```elixir\nconfig :temple,\n  engine: EEx.SmartEngine,\n  attributes: {Temple, :attributes}\n```\n\n> #### Note {: .info}\n>\n> This function does not do any HTML escaping\n\n> #### Note {: .info}\n>\n> This function is used by the compiler and shouldn't need to be used directly.","ref":"Temple.html#attributes/1","title":"Temple.attributes/1","type":"function"},{"doc":"","ref":"Temple.html#temple/1","title":"Temple.temple/1","type":"macro"},{"doc":"Use this module to create your own component implementation.\n\nThis is only required if you are not using a component implementation from another framework,\nlike Phoenix LiveView.\n\nAt it's core, a component implmentation includes the following functions\n\n- `component/2`\n- `inner_block/2`\n- `render_slot/2`\n\nThese functions are used by the template compiler, so you won't be calling them directly.","ref":"Temple.Component.html","title":"Temple.Component","type":"module"},{"doc":"Invoke the `__using__/1` macro to create your own module, and then import that module where you\nneed to define define or use components (usually everywhere).\n\nWe'll use an example that is similar to what Temple uses in its own test suite..\n\n```elixir\ndefmodule MyAppWeb.Component do\n  use Temple.Component\n\n  defmacro __using__(_) do\n    quote do\n      import Temple\n      import unquote(__MODULE__)\n    end\n  end\nend\n```\n\nThen you can `use` your module when you want to define or use a component.\n\n```elixir\ndefmodule MyAppWeb.Components do\n  use MyAppWeb.Component\n\n  def basic_component(_assigns) do\n    temple do\n      div do\n        \"I am a basic component\"\n      end\n    end\n  end\nend\n```","ref":"Temple.Component.html#module-usage","title":"Usage - Temple.Component","type":"module"},{"doc":"![Temple](temple-github-image.png)\n\n\n[![Actions Status](https://github.com/mhanberg/temple/workflows/CI/badge.svg)](https://github.com/mhanberg/temple/actions)\n[![Hex.pm](https://img.shields.io/hexpm/v/temple.svg)](https://hex.pm/packages/temple)\n\n> You are looking at the README for the main branch. The README for the latest stable release is located [here](https://github.com/mhanberg/temple/tree/v0.11.0).\n\n# Temple\n\nTemple is an Elixir DSL for writing HTML and SVG.","ref":"readme.html","title":"README","type":"extras"},{"doc":"Add `temple` to your list of dependencies in `mix.exs`:\n\n<!-- x-release-please-start-version -->\n```elixir\ndef deps do\n  [\n    {:temple, \"~> 0.12\"}\n  ]\nend\n```\n<!-- x-release-please-end -->","ref":"readme.html#installation","title":"Installation - README","type":"extras"},{"doc":"Currently Temple has the following things on which it won't compromise.\n\n- Will only work with valid Elixir syntax.\n- Should work in all web environments such as Plug, Aino, Phoenix, and Phoenix LiveView.","ref":"readme.html#goals","title":"Goals - README","type":"extras"},{"doc":"Using Temple is as simple as using the DSL inside of an `temple/1` block. The runtime result of the macro is your HTML.\n\nSee the [guides](https://hexdocs.pm/temple/your-first-template.html) for more details.\n\n```elixir\nimport Temple\n\ntemple do\n  h2 do: \"todos\"\n\n  ul class: \"list\" do\n    for item <- @items do\n      li class: \"item\" do\n        div class: \"checkbox\" do\n          div class: \"bullet hidden\"\n        end\n\n        div do: item\n      end\n    end\n  end\n\n  script do: \"\"\"\n  function toggleCheck({currentTarget}) {\n    currentTarget.children[0].children[0].classList.toggle(\"hidden\");\n  }\n\n  let items = document.querySelectorAll(\"li\");\n\n  Array.from(items).forEach(checkbox => checkbox.addEventListener(\"click\", toggleCheck));\n  \"\"\"\nend\n```","ref":"readme.html#usage","title":"Usage - README","type":"extras"},{"doc":"Temple components are simple to write and easy to use.\n\nUnlike normal partials, Temple components have the concept of \"slots\", which are similar [Vue](https://v3.vuejs.org/guide/component-slots.html#named-slots). You can also refer to HEEx and Surface for examples of templates that have the \"slot\" concept.\n\nTemple components are compatible with HEEx and Surface components and can be shared.\n\nPlease see the [guides](https://hexdocs.pm/temple/components.html) for more details.\n\n```elixir\ndefmodule MyAppWeb.Component do\n  import Temple\n\n  def card(assigns) do\n    temple do\n      section do\n        div do\n          slot @header\n        end\n\n        div do\n          slot @inner_block\n        end\n\n        div do\n          slot @footer\n        end\n      end\n    end\n  end\nend\n```\n\nUsing components is as simple as passing a reference to your component function to the `c` keyword.\n\n```elixir\nimport MyAppWeb.Component\n\nc &card/1 do\n  slot :header do\n    @user.full_name\n  end\n\n  @user.bio\n\n  slot :footer do\n    a href: \"https://twitter.com/#{@user.twitter}\" do\n      \"@#{@user.twitter}\"\n    end\n    a href: \"https://github.com/#{@user.github}\" do\n      \"@#{@user.github}\"\n    end\n  end\nend\n```","ref":"readme.html#components","title":"Components - README","type":"extras"},{"doc":"By default, Temple will use the `EEx.SmartEngine` that is built into the Elixir standard library. If you are a web framework that uses it's own template engine (such as [Aino](https://github.com/oestrich/aino) and Phoenix/LiveView, you can configure Temple to it!\n\n```elixir\n# config/config.exs\n\nconfig :temple,\n  engine: Aino.View.Engine # or Phoenix.HTML.Engine or Phoenix.LiveView.Engine\n```","ref":"readme.html#engine","title":"Engine - README","type":"extras"},{"doc":"To include Temple's formatter configuration, add `:temple` to your `.formatter.exs`.\n\n```elixir\n[\n  import_deps: [:temple],\n  inputs: [\"*.{ex,exs}\", \"priv/*/seeds.exs\", \"{config,lib,test}/**/*.{ex,exs,lexs}\"],\n]\n```","ref":"readme.html#formatter","title":"Formatter - README","type":"extras"},{"doc":"When using Phoenix ~> 1.7, all you need to do is include `:temple` in your mix.exs.\n\nIf you plan on using the template structure that < 1.6 Phoenix applications use, you can use `:temple_phoenix` as described below.\n\nTo use with [Phoenix](https://github.com/phoenixframework/phoenix), please use the [temple_phoenix](https://github.com/mhanberg/temple_phoenix) package! This bundles up some useful helpers as well as the Phoenix Template engine.","ref":"readme.html#phoenix","title":"Phoenix - README","type":"extras"},{"doc":"- [Introducing Temple: An elegant HTML library for Elixir and Phoenix](https://www.mitchellhanberg.com/introducing-temple-an-elegant-html-library-for-elixir-and-phoenix/)\n- [Temple, AST, and Protocols](https://www.mitchellhanberg.com/temple-ast-and-protocols/)\n- [Thinking Elixir Episode 92: Temple with Mitchell Hanberg](https://podcast.thinkingelixir.com/92)\n- [How EEx Turns Your Template Into HTML](https://www.mitchellhanberg.com/how-eex-turns-your-template-into-html/)","ref":"readme.html#related","title":"Related - README","type":"extras"},{"doc":"# Getting Started","ref":"getting-started.html","title":"Getting Started","type":"extras"},{"doc":"Welcome!\n\nTemple is a HTML DSL for Elixir, let's get started!\n\n\nFirst, make sure you are using Elixir `V1.13` or higher.\n\nAdd `:temple` to your deps and run `mix deps.get`\n\n```elixir\n{:temple, \"~> 0.9.0-rc.0\"}\n```\n\nNow you must prepend the Temple compiler to your projects `:compilers` configuration in `mix.exs`. There is a chance that your project doesn't set this option at all, but don't worry, it's really easy to add!\n\n```elixir\ndefmodule MyApp.MixProject do\n  use Mix.Project\n\n  def project do\n    [\n      # ...\n      compilers: [:temple] ++ Mix.compilers(),\n      # ...\n    ]\n  end\n\n# ...\n\nend\n```\n\nAll done, Now let's start building our app!","ref":"getting-started.html#install","title":"Install - Getting Started","type":"extras"},{"doc":"Temple works out of the box without any configuration, but here are a couple of conifg options that you could need to use.","ref":"getting-started.html#configuration","title":"Configuration - Getting Started","type":"extras"},{"doc":"By default, Temple uses the built in `Phoenix.HTML.Engine`. If you want to use a different engine, this is as easy as setting the `:engine` configuration option.\n\nYou can also configure the function that is used for runtime attributes. By default, Temple uses `Phoenix.HTML.attributes_escape/1`.\n\n```elixir\n# config/config.exs\n\nconfig :temple,\n  engine: EEx.SmartEngine,\n  attributes: {Temple, :attributes}\n```","ref":"getting-started.html#engine","title":"Engine - Getting Started","type":"extras"},{"doc":"Temple code will reserve some local function calls for HTML tags. If you have a local function that you would like to use instead, you can create an alias for any tag.\n\nCommon aliases for Phoenix projects look like this:\n\n```elixir\nconfig :temple,\n  aliases: [\n    label: :label_tag,\n    link: :link_tag,\n    select: :select_tag,\n    textarea: :textarea_tag\n  ]\n```","ref":"getting-started.html#aliases","title":"Aliases - Getting Started","type":"extras"},{"doc":"# Your First Template\n\nA Temple template is written inside of the `Temple.temple/1` macro. Code inside there will be compiled into efficient Elixir code by the configured EEx engine. \n\nLocal functions that have a corresponding HTML5 tag are reserved and will be used when generated your markup. Let's take a look at a basic form written with Temple.\n\n```elixir\ndefmodule MyApp.FormExample do\n  import Temple\n\n  def form_page() do\n    assigns = %{title: \"My Site | Sign Up\", logged_in: false}\n\n    temple do\n      \"<!DOCTYPE html>\"\n\n      html do\n        head do\n          meta charset: \"utf-8\"\n          meta http_equiv: \"X-UA-Compatible\", content: \"IE=edge\"\n          meta name: \"viewport\", content: \"width=device-width, initial-scale=1.0\"\n          link rel: \"stylesheet\", href: \"/css/app.css\"\n\n          title do: @title\n        end\n\n        body do\n          if @logged_in do\n            header class: \"header\" do\n              ul do\n                li do\n                  a href: \"/\", do: \"Home\"\n                end\n                li do\n                  a href: \"/logout\", do: \"Logout\"\n                end\n              end\n            end\n          end\n\n          form action: \"\", method: \"get\", class: \"form-example\" do\n            div class: \"form-example\" do\n              label for: \"name\", do: \"Enter your name:\"\n              input type: \"text\", name: \"name\", id: \"name\", required: true\n            end\n            div class: \"form-example\" do\n              label for: \"email\", do: \"Enter your email:\"\n              input type: \"email\", name: \"email\", id: \"email\", required: true\n            end\n            div class: \"form-example\" do\n              input type: \"submit\", value: \"Subscribe!\"\n            end\n          end\n        end\n      end\n    end\n  end\nend\n```\n\nThis example showcases an entire HTML page made with Temple! Let's dive a little deeper everything we're seeing here.\n\nThrough out this guide, you will see code that includes features that are explained later on. Feel free to skip ahead to read on, or just keep reading. It will all make sense eventually!","ref":"your-first-template.html","title":"Your First Template","type":"extras"},{"doc":"The text node is a basic building block of any HTML document. In Temple, text nodes are represented by Elixir string literals.\n\nThe very first line of the previous example is our doc type, emitted into the final document with `\"<!DOCTYPE html>\"`. This is a text node and will be emitted into the document as-is.\n\nNote: String _literals_ are emitted into text nodes. If you are using string interpolation with the `#{some_expression}` syntax, that is treated as an expression and will be evaluated in whichever way the configured engine evaluates expression. By default, the `EEx.SmartEngine` doesn't do any escaping of expressions, so that could still be emitted as-is, or even as HTML to be interpreted by your web browser.","ref":"your-first-template.html#text-nodes","title":"Text Nodes - Your First Template","type":"extras"},{"doc":"Void tags are HTML5 tags that do not have children, meaning they are \"self closing\".\n\nWe can observe these in the previous example as the ` ` tag. You'll note that the tag does not have a `:do` key or a `do` block.","ref":"your-first-template.html#void-tags","title":"Void Tags - Your First Template","type":"extras"},{"doc":"Non-void tags are HTML5 tags that _do_ have children. You are probably most familiar with these type of tags, as they include the famous `  ` and `  `.\n\nThese tags can enclose their children nodes with either a `do/end` block or the inline `:do` keyword.","ref":"your-first-template.html#non-void-tags","title":"Non-void Tags - Your First Template","type":"extras"},{"doc":"Nonvoid tags that use the `do/end` syntax will be emitted _with_ internal whitespace.\n\n```elixir\ntemple do\n  div class: \"foo\" do\n    # children\n  end\nend\n```\n\n...will emit markup that looks like...\n\n```html\n \n  <!-- children -->\n \n```\n\nNote: The Elixir comment _will not_ be rendered into an HTML comment. This is just used in the example. (This does sound like a good feature though...)\n\nNonvoid tags that use the `:do` keyword syntax will be emitted _without_ internal whitespace. This allows you to correctly use the `:empty` CSS psuedo-selector in your stylesheet. \n\n\n```elixir\ntemple do\n  p class: \"alert alert-info\", do: \"Your account was recently updated!\"\nend\n```\n\n...will emit markup that looks like...\n\n```html\n Your account was recently updated! \n```","ref":"your-first-template.html#whitespace","title":"Whitespace - Your First Template","type":"extras"},{"doc":"Temple leverages `Phoenix.HTML.attributes_escape/1` internally, so you can refer to it's documentation for all of the details.","ref":"your-first-template.html#attributes","title":"Attributes - Your First Template","type":"extras"},{"doc":"To render dynamic attributes into a tag, you can pass them with the reserved attribute `:rest!`.\n\n```elixir\nassigns = %{\n  data: [data_foo: \"hi\"]\n}\n\ntemple do\n  div id: \"foo\", rest!: @data do\n    \"Hello, world!\"\n  end\nend\n```\n\nwill render to\n\n```html\n \n  Hello, world!\n \n```","ref":"your-first-template.html#dynamic-attributes","title":"Dynamic Attributes - Your First Template","type":"extras"},{"doc":"Any Elixir expression can be used anywhere inside of a Temple template. Here are a few examples.\n\n```elixir\ntemple do\n  h2 do: \"Members\"\n\n  ul do\n    for member <- @members do\n      li do: member\n    end\n  end\nend\n```","ref":"your-first-template.html#elixir-expressions","title":"Elixir Expressions - Your First Template","type":"extras"},{"doc":"Match expression are handled slightly differently. Generally if you are assigning an expression to a variable (a match), you are going to use that binding later and do _not_ want to emit it into the document.\n\nSo, match expressions are _not_ emitted into the document. They are functionally equivalent to the `<% .. %.` syntax of `EEx`. The expression is evaluated, but not included in the rendered document.\n\nTypically you should not be writing this type of expression inside of your template, but if you wanted to declare an alias, you would need to write the following to not emit the alias into the document.\n\n```elixir\ntemple do\n  _ = alias My.Deep.Module\n\n  div do\n    Module.func()\n  end\nend\n```","ref":"your-first-template.html#match-expressions","title":"Match Expressions - Your First Template","type":"extras"},{"doc":"Since Temple uses the `EEx.SmartEngine` by default, you are able to use the assigns feature.\n\nThe assigns feature allows you to ergonomically access the members of a `assigns` variable by the `@` macro.\n\nThe assign variable just needs to exist within the scope of the template (the same as a normal `EEx` template that uses `EEx.SmartEngine`), it can be a function parameter or created inside the function.\n\n```elixir\ndef card(assigns) do\n  temple do\n    div class: \"card\" do\n      section class: \"card-header\" do\n        @name\n      end\n\n      section class: \"card-body\" do\n        @bio\n      end\n\n      if Enum.any?(@socials) do\n        section class: \"card-footer\" do\n          for social <- @socials do\n            a href: social.link do\n              social.name\n            end\n          end\n        end\n      end\n    end\n  end\nend\n```","ref":"your-first-template.html#assigns","title":"Assigns - Your First Template","type":"extras"},{"doc":"# Components\n\nTemple has the concept of components, which allow you an expressive and composable way to break up your templates into re-usable chunks.\n\nA component is any arity-1 function that take an argument called `assigns` and returns the result of the `Temple.temple/1` macro.","ref":"components.html","title":"Components","type":"extras"},{"doc":"Here is an example of a simple Temple component. You can observe that it seems very similar to a regular Temple template, and that is because it is a regular template!\n\n```elixir\ndefmodule MyApp.Components do\n  import Temple\n\n  def button(assigns) do\n    temple do\n      button type: \"button\", class: \"bg-blue-800 text-white rounded #{@class}\" do\n        @text\n      end\n    end\n  end\nend\n```","ref":"components.html#definition","title":"Definition - Components","type":"extras"},{"doc":"To use a component, you will use the special `c` keyword. This is called a \"keyword\" because it is not a function or macro, but only exists inside of the `Temple.temple/1` block.\n\nThe first argument will be the function reference to your component function, followed by any assigns. You can pass dynamic assigns using the `:rest!` keyword the same way you would with a normal tag.\n\n```elixir\ndefmodule MyApp.ConfirmDialog do\n  import Temple\n  import MyApp.Components\n\n  def render(assigns) do\n    temple do\n      dialog open: true do\n        p do: \"Are you sure?\"\n        form method: \"dialog\" do\n          c &button/1, class: \"border border-white\", text: \"Yes\"\n        end\n      end\n    end\n  end\nend\n```","ref":"components.html#usage","title":"Usage - Components","type":"extras"},{"doc":"Temple components can take \"slots\" as well. This is the method for providing dynamic content from the call site into the component.\n\nSlots are defined and rendered using the `slot` keyword. This is similar to the `c` keyword, in that it is not defined using a function or macro.","ref":"components.html#slots","title":"Slots - Components","type":"extras"},{"doc":"The default slot can be rendered from within your component by passing the `slot` the `@inner_block` assign. Let's redefine our button component using slots.\n\n```elixir\ndefmodule MyApp.Components do\n  import Temple\n\n  def button(assigns) do\n    temple do\n      button type: \"button\", class: \"bg-blue-800 text-white rounded #{@class}\" do\n        slot @inner_block\n      end\n    end\n  end\nend\n```\n\nYou can pass content through the \"default\" slot of your component simply by passing a `do/end` block to your component at the call site. This is a special case for the default slot.\n\n```elixir\ndefmodule MyApp.ConfirmDialog do\n  import Temple\n  import MyApp.Components\n\n  def render(assigns) do\n    temple do\n      dialog open: true do\n        p do: \"Are you sure?\"\n        form method: \"dialog\" do\n          c &button/1, class: \"border border-white\" do\n            \"Yes\"\n          end\n        end\n      end\n    end\n  end\nend\n```","ref":"components.html#default-slot","title":"Default Slot - Components","type":"extras"},{"doc":"You can also define a \"named\" slot, which allows you to pass more than one set of dynamic content to your component.\n\nWe'll use a \"card\" example to illustrate this. This example is adapted from the [Surface documentation](https://surface-ui.org/slots) on slots.\n\n#### Definition\n\n```elixir\ndefmodule MyApp.Components do\n  import Temple\n\n  def card(assigns) do\n    temple do\n      div class: \"card\" do\n        header class: \"card-header\", style: \"background-color: @f5f5f5\" do\n          p class: \"card-header-title\" do\n            slot @header\n          end\n        end\n\n        div class: \"card-content\" do\n          div class: \"content\" do\n            slot @inner_block\n          end\n        end\n\n        footer class: \"card-footer\", style: \"background-color: #f5f5f5\" do\n          slot @footer\n        end\n      end\n    end\n  end\nend\n```\n\n#### Usage\n\n```elixir\ndef MyApp.CardExample do\n  import Temple\n  import MyApp.Components\n\n  def render(assigns) do\n    temple do\n      c &card/1 do\n        slot :header do\n          \"A simple card component\"\n        end\n\n        \"This example demonstrates how to create components with multiple, named slots\"\n\n        slot :footer do\n          a href: \"#\", class: \"card-footer-item\", do: \"Footer Item 1\"\n          a href: \"#\", class: \"card-footer-item\", do: \"Footer Item 2\"\n        end\n      end\n    end\n  end\nend\n```","ref":"components.html#named-slots","title":"Named Slots - Components","type":"extras"},{"doc":"Sometimes it is necessary to pass data _into_ a slot (hereby known as *slot attributes*) from the call site and _from_ a component definition (hereby known as *slot arguments*) back to the call site. Dynamic slot attributes can be passed using the `:rest!` attribute in the same way you can with tag attributes.\n\nLet's look at what a `table` component could look like. Here we observe we access an attribute in the slot in the header with `col.label`.\n\nThis example is taken from the HEEx documentation to demonstrate how you can build the same thing with Temple.\n\nNote: Slot attributes can only be accessed on an individual slot, so if you define a single slot definition, you still need to loop through it to access it, as they are stored as a list.\n\n#### Definition\n\n```elixir\ndefmodule MyApp.Components do\n  import Temple\n\n  def table(assigns) do\n    temple do\n      table do\n        thead do\n          tr do\n            for col <- @col do\n              th do: col.label # 👈 accessing a slot attribute\n            end\n          end\n        end\n\n        tbody do\n          for row <- @rows do\n            tr do\n              for col <- @col do\n                td do\n                  slot col, row\n                end\n              end\n            end\n          end\n        end\n      end\n    end\n  end\nend\n```\n\n#### Usage\n\nWhen we render the slot, we can pattern match on the data passed through the slot via the `:let` attribute.\n\n```elixir\ndef MyApp.TableExample do\n  import Temple\n  import MyApp.Componens\n\n  def render(assigns) do\n    temple do\n      section do\n        h2 do: \"Users\"\n\n        c &table/1, rows: @users do\n          #          👇 defining the parameter for the slot argument\n          slot :col, let!: user, label: \"Name\" do # 👈 passing a slot attribute\n            user.name\n          end\n\n          slot :col, let!: user, label: \"Address\" do\n            user.address\n          end\n        end\n      end\n    end\n  end\nend\n```","ref":"components.html#passing-data-to-and-through-slots","title":"Passing data to and through Slots - Components","type":"extras"},{"doc":"# Converting HTML\n\nIf you want to use something like [TailwindUI](https://tailwindui.com) with Temple, you're going to have to convert a ton of vanilla HTML into Temple syntax.\n\nLuckily, Temple provides a mix task for converting an HTML file into Temple syntax and writes it to stdout.","ref":"converting-html.html","title":"Converting HTML","type":"extras"},{"doc":"First, we would want to create a temporary HTML file with the HTML we'd like to convert.\n\n> #### Hint {: .tip}\n>\n> The following examples use the `pbpaste` and `pbcopy` utilities found on macOS. These are used to send your clipboard contents into stdout and put stdout into your clipboard.\n\n```shell\n$ pbpaste > temp.html\n```\n\nThen, we can convert that file and copy the output into our clipboard.\n\n```shell\n$ mix temple.convert temp.html | pbcopy\n```\n\nNow, you are free to paste the new temple syntax into your project!","ref":"converting-html.html#usage","title":"Usage - Converting HTML","type":"extras"},{"doc":"# Migrating from 0.8 to 0.9\n\nFirst off, Temple now requires Elixir 1.13 or higher. This is because of some changes that were brought to the Elixir parser.","ref":"0-8-to-0-9.html","title":"Migrating from 0.8 to 0.9","type":"extras"},{"doc":"To control whitespace in an element, Temple will now control this based on whether the `do` was used in the keyword list syntax or the do/end syntax.\n\nIn 0.8, you would do:\n\n```elixir\nspan do\n  \"hello!\"\nend\n\n#  \n#   hello!\n#  \n\n# The ! version of the element would render it as \"tight\"\nspan! do\n  \"hello!\"\nend\n\n#  hello! \n```\n\nIn 0.9, you would do:\n\n```elixir\nspan do\n  \"hello!\"\nend\n\n#  \n#   hello!\n#  \n\nspan do: \"hello!\"\n\n#  hello! \n```","ref":"0-8-to-0-9.html#whitespace-control","title":"Whitespace Control - Migrating from 0.8 to 0.9","type":"extras"},{"doc":"Components are no longer module based. To render a component, you can pass a function reference to the `c` keyword. You also no longer need to define a component in a module, using the `Temple.Component` module and its `render` macro.\n\nIn 0.8, you would define a component like:\n\n```elixir\ndefmodule MyAppWeb.Component.Card do\n  import Temple.Component\n\n  render do\n    div class: \"border p-4 rounded\" do\n      slot :default\n    end\n  end\nend\n```\n\nAnd you would use the component like:\n\n```elixir\ndiv do\n  c MyAppWeb.Component.Card do\n    \"Welcome to my app!\"\n  end\nend\n```\n\nIn 0.9, you would define a component like:\n\n```elixir\ndefmodule MyAppWeb.Components do\n  import Temple\n\n  def card(assigns) do\n    temple do\n      div class: \"border p-4 rounded\" do\n        slot :default\n      end\n    end\n  end\nend\n```\n\nAnd you would use the component like:\n\n```elixir\ndiv do\n  c &MyAppWeb.Components.card/1 do\n    \"Welcome to my app!\"\n  end\nend\n```\n\nWe can observe here that in 0.9 the component is just any 1-arity function, so you can define them anywhere and you can have more than 1 in a single module.","ref":"0-8-to-0-9.html#components","title":"Components - Migrating from 0.8 to 0.9","type":"extras"},{"doc":"Now that components are just functions, you no longer need this special macro to define a component in the middle of the module.\n\nThis can simply be converted to a function.","ref":"0-8-to-0-9.html#defcomp","title":"defcomp - Migrating from 0.8 to 0.9","type":"extras"},{"doc":"All Phoenix related items have moved to the [temple_phoenix](https://github.com/mhanberg/temple_phoenix) package. Please see that library docs for more details.","ref":"0-8-to-0-9.html#phoenix","title":"Phoenix - Migrating from 0.8 to 0.9","type":"extras"},{"doc":"# Migrating from 0.10 to 0.11\n\nMost of the changes in this release are related to tweaking Temple's component model to align with HEEx & Surface.","ref":"0-10-to-0-11.html","title":"Migrating from 0.10 to 0.11","type":"extras"},{"doc":"Slots are now available as assigns in the component and are rendered as such.","ref":"0-10-to-0-11.html#rendering-slots","title":"Rendering Slots - Migrating from 0.10 to 0.11","type":"extras"},{"doc":"```elixir\ndef my_component(assign) do\n  temple do\n    span do\n      slot :a_slot\n    end\n  end\nend\n```","ref":"0-10-to-0-11.html#before","title":"Before - Migrating from 0.10 to 0.11","type":"extras"},{"doc":"```elixir\ndef my_component(assign) do\n  temple do\n    span do\n      slot @a_slot\n    end\n  end\nend\n```\n\n## :default slot has been renamed to :inner_block\n\nThe main body of a component has been renamed from `:default` to `:inner_block`.\n\nNote: The \"after\" example also includes the necessary change specified above.","ref":"0-10-to-0-11.html#after","title":"After - Migrating from 0.10 to 0.11","type":"extras"},{"doc":"```elixir\ndef my_component(assign) do\n  temple do\n    span do\n      slot :default\n    end\n  end\nend\n```","ref":"0-10-to-0-11.html#before","title":"Before - Migrating from 0.10 to 0.11","type":"extras"},{"doc":"```elixir\ndef my_component(assign) do\n  temple do\n    span do\n      slot @inner_block\n    end\n  end\nend\n```","ref":"0-10-to-0-11.html#after","title":"After - Migrating from 0.10 to 0.11","type":"extras"},{"doc":"The syntax for capturing data being passed from the call site of a slot to the definition of a slot (or put another way, from the definition of a component to the call site of the component) has changed. \n\nYou now capture it as the value of the `:let!` attribute on the slot definition.","ref":"0-10-to-0-11.html#passing-data-into-slots","title":"Passing data into slots - Migrating from 0.10 to 0.11","type":"extras"},{"doc":"```elixir\ndef my_component(assign) do\n  temple do\n    c &my_component/1 do\n      slot :a_slot, %{some: value} do\n        \"I'm using some #{value}\"\n      end\n    end\n  end\nend\n```","ref":"0-10-to-0-11.html#before","title":"Before - Migrating from 0.10 to 0.11","type":"extras"},{"doc":"```elixir\ndef my_component(assign) do\n  temple do\n    c &my_component/1 do\n      slot :a_slot, let!: %{some: value} do\n        \"I'm using some #{value}\"\n      end\n    end\n  end\nend\n```","ref":"0-10-to-0-11.html#after","title":"After - Migrating from 0.10 to 0.11","type":"extras"}],"producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}